﻿using AdventOfCode.Common;
using AdventOfCode.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace AdventOfCode._2023
{
    public class Day4 : AOCDay
    {
      
        protected override AOCResponse ExecutePartA()
        {
            var cards = new List<Card>();
            foreach (var entry in this.GetSplitInput())
            {
                cards.Add(new Card(entry));
            }

            foreach (var card in cards)
            {
                Log($"Card {card.Id} has a score of {card.GenerateScore()}");
            }

            this.Answer = cards.Sum(x => x.GenerateScore());
            return this._response;
        }

        protected override AOCResponse ExecutePartB()
        {

            var cards = new List<Card>();
            foreach (var entry in this.GetSplitInput())
            {
                cards.Add(new Card(entry));
            }

            foreach (var card in cards)
            {
                for (int cardCount = 0; cardCount < card.CardCount; cardCount++)
                {
                    var count = card.GetCountOfWinningNumbers();
                    for (int i = 1; i <= count; i++)
                    {
                        cards[(card.Id + i)-1].CardCount++;
                    }
                }                
            }

            this.Answer = cards.Sum(x => x.CardCount);

            return this._response;
        }
    }

    class Card
    {
        public int Id { get; set; }
        public int CardCount { get; set; }
        public List<int> WinningNumbers { get; set; } = new List<int>();
        public List<int> Numbers { get; set; } = new List<int>();
        public Card(string entry)
        {
            CardCount = 1;
            entry = entry.Replace("   ", "  ").Replace("  ", " "); //Replace two spaces with 1 
            var split = entry.Split(':');
            Id = int.Parse(split[0].Split(' ')[1]);

            //Winning numbers
            foreach (var number in split[1].Split('|')[0].Trim().Split(' '))
            {
                WinningNumbers.Add(int.Parse(number));
            }

            //Winning numbers
            foreach (var number in split[1].Split('|')[1].Trim().Split(' '))
            {
                Numbers.Add(int.Parse(number));
            }
        }

        public int GenerateScore()
        {
            var score = 0;
            foreach (var number in Numbers)
            {
                if (WinningNumbers.Contains(number))
                {
                    if (score == 0)
                    {
                        score = 1;
                    }
                    else
                    {
                        score *= 2;
                    }
                }
            }
            return score;
        }

        public int GetCountOfWinningNumbers()
        {
            var count = 0;
            foreach (var number in Numbers)
            {
                if (WinningNumbers.Contains(number))
                {
                    count++;
                }
            }
            return count;
        }
    }
}
